unit plwreadermain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, DateUtils, StrUtils, ShellApi;

const WM_AFTER_SHOW = WM_USER + 300;

type
  TFormMain = class(TForm)
    Memo1: TMemo;
    procedure FormShow(Sender: TObject);
    procedure ButtonStartClick(Sender: TObject);
  private
    { Private-Deklarationen }
    procedure WMDROPFILES(var msg : TWMDropFiles) ; message WM_DROPFILES;
    procedure WmAfterShow(var Msg: TMessage); message WM_AFTER_SHOW;
    procedure ReadPLW(FileName : String);
  public
    { Public-Deklarationen }
  end;

type PlwHeader = packed record
  HeaderSize : word; // Length, in bytes, of this header
  Signature : Array[1..40] of AnsiChar; // "PicoLog for Windows"
  Version : longword;
  NoOfParameters : longword; // Number of parameters recorded
  Parameters : array[1..250] of word;
  SampleNo : longword; // Same as no_of_samples, unless wraparound occurred
  NoOfSamples : longword; // Number of samples recorded so far
  MaxSample : longword;
  Interval : longword;
  IntervalUnit : word; // 0=femtoseconds, 4=milliseconds, 5=seconds, 6=minutes, 7=hours
  TriggerSample : longword;
  Triggered : word;
  FirstSample : longword;
  SampleByte : longword;    // Length of each sample record
  SettingsBytes : longword;  // Length of settings text aftersamples (copy of .pls file)
  StartDate : longword;  // days since start of year 0
  StartTime : longword; // the number of seconds elapsed in one day (starting at midnight).
  MinTime : longword;
  MaxTime : longword;
  Notes : Array[1..1000] of AnsiChar;
  CurrentTime : longword;
  StopAfter : word;
  MaxTimeUnit :word;
  MaxSampleTime : longword;
  StartTimeMsAccuracy : longword; // Time when sampling starts in real-time collection mode.
  PreviousTimeMsAccuracy : longword; // Time when the last sample was taken in real-time mode.
  NoOfDays : LongWord; // The number of days elapsed since sampling in real-time collection mode started.
  Spare : array[1..58] of byte;
end;

var FormMain: TFormMain;
    MySqlFormatSettings : TFormatSettings;

implementation

{$R *.dfm}

procedure TFormMain.ReadPLW(FileName : String);
var f : file;
    fPos,i,j : LongWord;
    Header : PlwHeader;
    mNo,Sec : Longword;
    mVal : single;
    s : String;
    d : TDateTime;

begin
  AssignFile(f,FileName);
  FileMode := fmOpenRead or fmShareDenyWrite;
  {$I-}
  Reset(f,1);
  {$I+}
  i := IOResult;
  if i<> 0 then begin
    ShowMessage('Error opening file, error code: ' + IntToStr(i));
    exit;
  end;
  BlockRead(f,Header,sizeof(Header));
  if LeftStr(Header.Signature,19) <> 'PicoLog for Windows' then begin
    ShowMessage('This is not a valid PLW file');
    exit;
  end;
  fPos := Header.HeaderSize;
  Sec := 1;
  case Header.IntervalUnit of
    (*
    0=femtoseconds
    4=milliseconds
    *)
    5 : Sec := Sec * 1; // seconds
    6 : Sec := Sec * 60; // minutes
    7 : Sec := Sec * 60*60; // hours
  end;

  Memo1.Clear;
  Memo1.Refresh;
  d := Header.StartDate-DateDelta; // 693594
  d := IncSecond(d,Header.StartTime);
  Screen.Cursor := crHourGlass;
  Memo1.Lines.Add('Start:' + DateTimeToStr(d,MySqlFormatSettings));
  Memo1.Lines.BeginUpdate;
  for j := 1 to Header.NoOfSamples do begin
    Application.ProcessMessages;
    Seek(f,fPos);
    BlockRead(f,mNo,sizeof(mNo));
    inc(fPos,SizeOf(mNo));
    s := '"' + Trim(DateTimeToStr(d,MySqlFormatSettings)) + '",' + IntToStr(mNo) + ',';
    for i := 1 to Header.NoOfParameters do begin
      Seek(f,fPos);
      BlockRead(f,mVal,sizeof(mVal));
      inc(fPos,SizeOf(mVal));
      if mVal < 0 then
        mVal := 0;
      s := s + FormatFloat('0.00',mVal,MySqlFormatSettings);
      if i <  Header.NoOfParameters then
        s := s + ',';
    end;
    d := IncSecond(d,Sec * Header.Interval);
    Memo1.Lines.Add(s);
  end;
  Memo1.Lines.EndUpdate;
  Screen.Cursor := crDefault;
  closefile(f);
end;

procedure TFormMain.WmAfterShow(var Msg: TMessage);
begin
  if ParamStr(1) <> '' then
    ReadPLW(ParamStr(1));
end;

procedure TFormMain.WMDROPFILES(var msg: TWMDropFiles);
const MAXFILENAME = 255;
var i, fileCount : integer;
    FileName : array [0..MAXFILENAME] of char;
begin
  fileCount := DragQueryFile(msg.Drop, $FFFFFFFF, FileName, MAXFILENAME);
  for i := 0 to fileCount-1 do begin
    DragQueryFile(msg.Drop, i, FileName, MAXFILENAME);
    if LowerCase(ExtractFileExt(FileName)) = '.plw' then
      ReadPLW(FileName);
  end;
  DragFinish(msg.Drop);
end;

procedure TFormMain.ButtonStartClick(Sender: TObject);
begin
  ReadPLW('');
end;

procedure TFormMain.FormShow(Sender: TObject);
begin
  DragAcceptFiles(Handle, True);
  Memo1.Clear;
  Memo1.Lines.Add('Use drag & drop to select files');
  MySqlFormatSettings.TimeSeparator := ':';
  MySqlFormatSettings.DecimalSeparator := '.';
  MySqlFormatSettings.ShortDateFormat := 'YYYY-MM-DD hh:nn:ss';
  MySqlFormatSettings.LongDateFormat := 'YYYY-MM-DD hh:nn:ss';
  PostMessage(Self.Handle, WM_AFTER_SHOW, 0, 0);
end;

end.
